@@ -100,10 +100,35 @@ void checkEntities(datablock &db)
 
 	for (auto entity : db["entity"].find("formula_weight"_key == null or "formula_weight"_key == 0))
 	{
-		const auto &[entity_id, type] = entity.get<std::string, std::string>("id", "type");
+		auto &&[entity_id, type] = entity.get<std::string, std::string>("id", "type");
 
 		float formula_weight = 0;
 
+		if (type.empty())	// yes, that happens
+		{
+			const auto comp_id = db["atom_site"].find_first<std::string>("label_entity_id"_key == entity_id, "label_comp_id");
+			auto compound = cf.create(comp_id);
+			if (compound != nullptr)
+			{
+				if (compound->is_base() or compound->is_peptide())
+					type = "polymer";
+				else if (compound->is_water())
+					type = "water";
+				else
+				{
+					if (db["pdbx_entity_branch_link"].contains("entity_id"_key == entity_id))
+						type = "branched";
+					else
+						type = "non-polymer";
+				}
+			}
+
+			if (type.empty())
+				throw std::runtime_error("Entity without type and cannot determine what it should be");
+			
+			entity["type"] = type;
+		}
+		
 		if (type == "polymer")
 		{
 			int n = 0;
@@ -1012,6 +1037,10 @@ void comparePolySeqSchemes(datablock &db)
 	auto &ndb_poly_seq_scheme = db["ndb_poly_seq_scheme"];
 	auto &pdbx_poly_seq_scheme = db["pdbx_poly_seq_scheme"];
 
+	// Don't bother if ndb_poly_seq_scheme is empty
+	if (ndb_poly_seq_scheme.empty())
+		return;
+
 	// Since often ndb_poly_seq_scheme only contains an id and mon_id item
 	// we assume that it should match the accompanying pdbx_poly_seq
 
@@ -1080,7 +1109,101 @@ void comparePolySeqSchemes(datablock &db)
 	}
 }
 
-bool reconstruct_pdbx(file &file, std::string_view dictionary)
+void createPdbxEntityNonpoly(datablock &db)
+{
+	using namespace literals;
+
+	auto &atom_site = db["atom_site"];
+	auto &entity = db["entity"];
+	auto &pdbx_entity_nonpoly = db["pdbx_entity_nonpoly"];
+
+	for (const auto &[entity_id, type] : entity.find<std::string, std::string>("type"_key == "water" or "type"_key == "non-polymer", "id", "type"))
+	{
+		for (auto comp_id : atom_site.find<std::string>("label_entity_id"_key == entity_id, "label_comp_id"))
+		{
+			if (auto test_comp_id = pdbx_entity_nonpoly.find_first<std::optional<std::string>>("entity_id"_key == entity_id, "comp_id"); test_comp_id.has_value())
+			{
+				if (*test_comp_id != comp_id)
+					throw std::runtime_error("Inconsistent pdbx_entity_nonpoly record for entity " + entity_id + ", expected comp_id " + comp_id);
+
+				continue;
+			}
+
+			if (type == "water")
+				pdbx_entity_nonpoly.emplace({ //
+					{ "entity_id", entity_id },
+					{ "name", "water" },
+					{ "comp_id", comp_id }
+				});
+			else
+			{
+				auto c = cif::compound_factory::instance().create(comp_id);
+
+				std::string name = c ? c->name() : ".";
+
+				pdbx_entity_nonpoly.emplace({ //
+					{ "entity_id", entity_id },
+					{ "name", name },
+					{ "comp_id", comp_id }
+				});
+			}
+		}
+	}
+}
+
+void createPdbxNonpolyScheme(datablock &db)
+{
+	using namespace literals;
+
+	createPdbxEntityNonpoly(db);
+
+	auto &pdbx_entity_nonpoly = db["pdbx_entity_nonpoly"];
+	auto &pdbx_nonpoly_scheme = db["pdbx_nonpoly_scheme"];
+	auto &atom_site = db["atom_site"];
+
+	for (const auto &[entity_id, comp_id] : pdbx_entity_nonpoly.rows<std::string,std::string>("entity_id", "comp_id"))
+	{
+		for (int ndb_nr = 1; auto row : atom_site.find("label_entity_id"_key == entity_id and "label_comp_id"_key == comp_id))
+		{
+			// Skip existing records
+			auto linked = atom_site.get_linked(row, pdbx_nonpoly_scheme);
+			if (not linked.empty())
+				continue;
+
+			int num = row.get<int>("auth_seq_id");
+
+			pdbx_nonpoly_scheme.emplace({//
+
+				{ "asym_id", row.get<std::string>("label_asym_id") },
+				{ "entity_id", entity_id },
+				{ "mon_id", comp_id },
+				{ "ndb_seq_num", ndb_nr++ },
+				{ "pdb_seq_num", num },
+				{ "auth_seq_num", num },
+				{ "pdb_mon_id", row.get<std::string>("auth_comp_id") },
+				{ "auth_mon_id",  row.get<std::string>("auth_comp_id") },
+				{ "pdb_strand_id", row.get<std::string>("auth_asym_id") },
+				{ "pdb_ins_code", row.get<std::string>("pdbx_PDB_ins_code") }
+
+			});
+		}
+	}
+}
+
+bool reconstruct_pdbx(file &file)
+{
+	if (file.empty())
+		throw std::runtime_error("Cannot reconstruct PDBx, file seems to be empty");
+
+	auto &db = file.front();
+
+	if (auto ac = db.get("audit_conform"); ac != nullptr)
+		return reconstruct_pdbx(file, validator_factory::instance().get(*ac));
+	else
+		return reconstruct_pdbx(file, validator_factory::instance().get("mmcif_pdbx.dic"));
+}
+
+bool reconstruct_pdbx(file &file, const validator &validator)
 {
 	if (file.empty())
 		throw std::runtime_error("Cannot reconstruct PDBx, file seems to be empty");
@@ -1094,8 +1217,6 @@ bool reconstruct_pdbx(file &file, std::string_view dictionary)
 	if (auto cat = db.get("atom_site"); cat == nullptr or cat->empty())
 		throw std::runtime_error("Cannot reconstruct PDBx file, atom data missing");
 
-	auto &validator = validator_factory::instance()[dictionary];
-
 	std::string entry_id;
 
 	// Phenix files do not have an entry record
@@ -1346,7 +1467,7 @@ bool reconstruct_pdbx(file &file, std::string_view dictionary)
 
 	db["chem_comp"].reorder_by_index();
 
-	file.load_dictionary(dictionary);
+	db.set_validator(&validator);
 
 	if (db.get("atom_site_anisotrop"))
 		checkAtomAnisotropRecords(db);
@@ -1367,13 +1488,15 @@ bool reconstruct_pdbx(file &file, std::string_view dictionary)
 
 	if (auto cat = db.get("ndb_poly_seq_scheme"); cat == nullptr or cat->empty())
 		comparePolySeqSchemes(db);
+	
+	createPdbxNonpolyScheme(db);
 
 	// skip unknown categories for now
 	bool valid = true;
 	for (auto &cat : db)
 		valid = valid and (cat.get_cat_validator() == nullptr or cat.is_valid());
 
-	return valid and is_valid_pdbx_file(file, dictionary);
+	return valid and is_valid_pdbx_file(file, validator);
 }
 
 } // namespace cif::pdb
